indexing
   description: "gdk color"
   status: "You may be able to redistribute and/or modify this program.% 
   % See the README for details."
   copyright: "Copyright (C) 1999 Paul C. Janzen"
   author: "Paul C. Janzen"
   id: "$Id: gdk_color.e,v 1.5 1999/01/18 19:54:55 pcj Exp $"

	 -- modified by Miron Brezuleanu, 1999/03/15
	 --          added support for allocation and creation of 
	 --          pixel color from r,g,b triplets
   
	 -- modified by Miron Brezuleanu, 1999/03/17
	 --          added suport for creation of r,g,b triplets from 
	 --          pixel color
	 --
	 --          GDK_COLOR.pixel added - just gets the pixel 
	 --          value from the color

class GDK_COLOR

inherit
   GDK_OBJECT
   EXG_CONSTANTS
   GTK_MAIN

creation
   make_default,
   make_from_pixel,
   exg_create,
   make,
   make_combination,
   make_combination2,
   make_from_hsv

feature {ANY} -- Commands
   
   make_from_hsv(h, s, v: DOUBLE) is
	 -- will build the color using the Hue, Saturation and Value
	 -- components
      
	 -- `hue' goes from 0 to 360 and is measured in degrees
      
	 -- `s' and `v' both range from 0 to 1
      require
	 h >= 0 and h <= 360
	 s >= 0 and s <= 1
	 v >= 0 and v <= 1
      local
	 i: INTEGER
	 f, p, q, t, hue_2: DOUBLE
	 r, g, b: INTEGER
      do
	 hsv_computed := True
	 hue := h
	 saturation := s
	 value := v
	 hue_2 := h
	 if hue_2 = 360 then 
	    hue_2 := 0.0
	 end
	 hue_2 := hue_2 / 60.0
	 i := hue_2.floor
	 f := hue_2 - i.to_double
	 p := v * (1 - s)
	 q := v * (1 - (s * f))
	 t := v * (1 - (s * (1 - f)))
	 inspect i
	 when 0 then
	    r := (v * 65536.0).floor 
	    g := (t * 65536.0).floor 
	    b := (p * 65536.0).floor
	 when 1 then
	    r := (q * 65536.0).floor
	    g := (v * 65536.0).floor
	    b := (p * 65536.0).floor
	 when 2 then
	    r := (p * 65536.0).floor
	    g := (v * 65536.0).floor
	    b := (t * 65536.0).floor
	 when 3 then
	    r := (p * 65536.0).floor
	    g := (q * 65536.0).floor
	    b := (v * 65536.0).floor
	 when 4 then
	    r := (t * 65536.0).floor
	    g := (p * 65536.0).floor
	    b := (v * 65536.0).floor
	 when 5 then
	    r := (v * 65536.0).floor
	    g := (p * 65536.0).floor
	    b := (q * 65536.0).floor
	 end
	 make(r, g, b)
      ensure
	 hsv_computed = True
      end
   
   make_combination2(col: COLLECTION[GDK_COLOR]; 
		     weights: COLLECTION[DOUBLE]) is
	 -- like `make_combination', but uses `weights' 
	 -- to average the resulting color
      local
         sum: DOUBLE
	 r, g, b: DOUBLE
	 i, i_r, i_g, i_b: INTEGER
      do
         from
	    i := weights.lower
	 until
	    i > weights.upper	    
	 loop
	    r := r + col.item(i).red.to_double * weights.item(i)
	    g := g + col.item(i).green.to_double * weights.item(i)
	    b := b + col.item(i).blue.to_double * weights.item(i)
	    sum := sum + weights.item(i)
	    i := i + 1
	 end
	 i_r := (r / sum).rounded
	 i_g := (g / sum).rounded
	 i_b := (b / sum).rounded
	 make(i_r, i_g, i_b)
      end

   make_combination(col: COLLECTION[GDK_COLOR]) is
	 -- combines all these colors into the current one
      local
	 i: INTEGER
	 r, g, b: DOUBLE
	 i_r, i_g, i_b: DOUBLE
      do
	 from
	    i := col.lower
	 until
	    i > col.upper
	 loop
	    r := r + (col @ i).red
	    g := g + (col @ i).green
	    b := b + (col @ i).blue
	    i := i + 1
	 end
	 i_r := r / col.count
	 i_g := g / col.count
	 i_b := b / col.count
	 make(i_r.rounded, i_g.rounded, i_b.rounded)
      end

   make_from_pixel(p: INTEGER) is
      -- buils a GdkColor in GC-able space from the pixel component
      do
	 !!storage.make(ext_sizeof_gdk_color)	 
	 gdkobject := storage.to_external
	 ext_new_color_from_pixel(gdkobject, p)
      end

   make(r, g, b: INTEGER) is
	 -- builds a GdkColor in GC-able space from r, g, b components
      do
	 !!storage.make(ext_sizeof_gdk_color)	 
	 gdkobject := storage.to_external
	 if application_colormap /= Void then
	    ext_new_color(gdkobject, r, g, b, 
			  application_colormap.to_external)
	 else
	    ext_new_color(gdkobject, r, g, b,
			  default_pointer)
	 end
      end
   
   compute_hsv is
	 -- this is to be called when one wants to get the HSV values 
	 -- of this color
      local
	 max, min, dr, dg, db: DOUBLE
	 r, g, b: DOUBLE
      do
	 r := red.to_double / 65536.0
	 g := green.to_double / 65536.0
	 b := blue.to_double / 65536.0
	 if r > g then max := r else max := g end
	 if b > max then max := b end 
	 if r < g then min := r else min := g end
	 if b < min then min := b end
	 value := max
	 if max /= 0.0 then 
	    saturation := (max - min)/max 
	 else 
	    saturation := 0 
	 end
	 if saturation /= 0.0 then 
	    dr := (max - r) / (max - min)
	    dg := (max - g) / (max - min)
	    db := (max - b) / (max - min)
	    if r = max then hue := db - dg
	    elseif g = max then hue := 2 + dr - db
	    elseif b = max then hue := 4 + dg - dr
	    end
	    hue := hue * 60.0
	    if hue < 0 then hue := hue + 360 end
	 end
	 hsv_computed := True
      ensure
	 hsv_computed = True
      end

feature {NONE} -- storage space
   
   storage: FIXED_ARRAY[CHARACTER]

feature {ANY} -- Queries
   
   hsv_computed: BOOLEAN
	 -- if this is false the `hue', `saturation' and `value' 
	 -- aren't computed so it would be wrong to use them
   
   hue, saturation, value: DOUBLE
   
   -- these queries should be faster (maybe some attributes, knowing 
   -- that colors are immutable?) 
   -- but who needs to read the value of a color?

   variation(d: DOUBLE): GDK_COLOR is
	 -- creates a darker or lighter color based on the current 
	 -- color, multiplying the RGB triplet with `d'
      do
	 !!Result.make((red.to_double * d).rounded, 
		       (green.to_double * d).rounded, 
		       (blue.to_double * d).rounded)
      end

   red : INTEGER is
      require
	 gdkobject /= default_pointer
      do
	 Result := ext_color_red ( gdkobject )
      end

   green : INTEGER is
      require
	 gdkobject /= default_pointer
      do
	 Result := ext_color_green ( gdkobject )
      end

   blue : INTEGER is
      require
	 gdkobject /= default_pointer
      do
	 Result := ext_color_blue ( gdkobject )
      end

   pixel: INTEGER is
      require
	 gdkobject /= default_pointer
      do
	 Result := ext_color_pixel(gdkobject)
      end
   
feature {NONE} -- Private
   
   ext_color_red ( color : POINTER ) : INTEGER is
      external "C"
      end
   ext_color_green ( color : POINTER ) : INTEGER is
      external "C"
      end
   ext_color_blue ( color : POINTER ) : INTEGER is
      external "C"
      end
   ext_color_pixel(color: POINTER) : INTEGER is
      external "C"
      end
   ext_sizeof_gdk_color : INTEGER is
      external "C"
      end
   ext_new_color(p: POINTER; r, g, b: INTEGER; p2: POINTER) is
      external "C"
      end  
   ext_new_color_from_pixel(p: POINTER; pix_color: INTEGER) is
      external "C"
      end  
   
invariant
end
