
indexing
   author: "Miron Brezuleanu"
   purpose: "deferred class to help wrap drawable functions in GDK_PIXMAP,%
   % GDK_WINDOW and GTK_DRAWING_AREA"
   
deferred class GDK_DRAWABLE_WRAPPER

inherit
   EXG_CONSTANTS
   GTK_CONSTANTS

feature  -- internal stuff, made public because of `require' clauses

   drawable: GDK_DRAWABLE
   drawable_is_computed: BOOLEAN

   internal_to_drawable: GDK_DRAWABLE is
	 -- function that returns a conversion of the current object 
	 -- to a drawable; the only deferred feature
      deferred
      end
   
feature -- Commands

   draw_point(gc: GDK_GC; x, y: INTEGER) is
      do
	 to_drawable.draw_point(gc, x, y)
      end

   draw_line(gc: GDK_GC; x1, y1, x2, y2: INTEGER) is
      do
	 to_drawable.draw_line(gc, x1, y1, x2, y2)
      end

   draw_rectangle(gc : GDK_GC; filled : BOOLEAN; 
		  x, y, width, height : INTEGER) is
      do
	 to_drawable.draw_rectangle(gc, filled, x, y, width, height)
      end
   
   draw_arc(gc : GDK_GC; filled : BOOLEAN; 
	    x, y, width, height, angle1, angle2 : INTEGER) is
      do
	 to_drawable.draw_arc(gc, filled, x, y, width, height, angle1, angle2)
      end

   draw_polygon(gc : GDK_GC; filled : BOOLEAN; 
		points : COLLECTION[GDK_POINT] ) is
      do
	 to_drawable.draw_polygon(gc, filled, points)
      end

   draw_string(font : GDK_FONT; gc : GDK_GC; x, y : INTEGER; text : STRING) is
      do
	 to_drawable.draw_string(font, gc, x, y, text)
      end
   
   draw_centered_string(font: GDK_FONT; gc: GDK_GC; 
			x, y: INTEGER;
			text: STRING) is
	 -- this is not in Gtk+ !!; this will draw the string 
	 -- centered at (x, y)
      local
	 ax, ay: INTEGER
	 h: INTEGER
	 gfe: GDK_FONT_EXTENTS
      do
	 gfe := font.string_extents(text)
	 ax := x - gfe.width // 2
	 h := gfe.ascent + gfe.descent
	 ay := y + h // 2 - gfe.descent
	 draw_string(font, gc, ax, ay, text)
      end

   -- What does this do? 
   draw_text(font : GDK_FONT; gc : GDK_GC; 
	     x, y : INTEGER; text : STRING; text_length : INTEGER) is
      do
	 to_drawable.draw_text(font, gc, x, y, text, text_length)
      end
   
   bitblt (gc: GDK_GC; source: GDK_DRAWABLE_WRAPPER; xsrc, ysrc,
	   xdest, ydest, width, height: INTEGER) is
	 -- same as `draw_pixmap' but uses a GDK_DRAWABLE_WRAPPER as a
	 -- source and, of course, doesn't worry about the mask; 
	 -- anyway, the name should say everything
      do
	 to_drawable.draw_pixmap(gc, source.to_drawable,
				 xsrc, ysrc,
				 xdest, ydest,
				 width, height)
      end

   draw_pixmap(gc : GDK_GC; source : GDK_PIXMAP; xsrc, ysrc, 
	       xdest, ydest, width, height : INTEGER) is
      do
	 if source.mask /= Void then
	    draw_pixmap_with_mask(gc, source, source.mask, xsrc, ysrc,
				  xdest, ydest, width, height)
	 else
	    to_drawable.draw_pixmap(gc, source.to_drawable, 
				    xsrc, ysrc,
				    xdest, ydest, 
				    width, height)
	 end
      end

   draw_pixmap_with_mask(gc : GDK_GC; 
			 source : GDK_PIXMAP; mask: GDK_BITMAP;
			 xsrc, ysrc, xdest, ydest, width, height : INTEGER) is
      require
	 mask /= Void
      do
	 gc.set_clip_mask(mask)
	 gc.set_clip_origin(xdest, ydest)	 
	 to_drawable.draw_pixmap(gc, source.to_drawable, 
				 xsrc, ysrc,
				 xdest, ydest, 
				 width, height)
	 gc.set_clip_mask(Void)
	 gc.set_clip_origin(0, 0)
      end

   draw_image(gc : GDK_GC; image : GDK_IMAGE; 
	      xsrc, ysrc, xdest, ydest, width, height : INTEGER) is
      do
	 to_drawable.draw_image(gc, image, 
				xsrc, ysrc, 
				xdest, ydest, width, height)
      end

   draw_points(gc : GDK_GC; points : COLLECTION[GDK_POINT] ) is
      do
	 to_drawable.draw_points(gc, points)
      end

   draw_segments(gc : GDK_GC; segs : COLLECTION[GDK_SEGMENT] ) is 
      do
	 to_drawable.draw_segments(gc, segs)
      end
   
   draw_lines(gc : GDK_GC; points : COLLECTION[GDK_POINT]) is
      do
	 to_drawable.draw_lines(gc, points)
      end

   to_drawable: GDK_DRAWABLE is
	 -- the external interface to `internal_to_drawable'; that is 
	 -- now to be defined by descendants
      do
	 if not drawable_is_computed then
	    drawable := internal_to_drawable
	    drawable_is_computed := True
	 end
	 Result := drawable
      end

end
