indexing
   description: "gdk image"
   status: "You may be able to redistribute and/or modify this program.% 
   % See the README for details."
   copyright: "Copyright (C) 1999 Paul C. Janzen"
   author: "Paul C. Janzen"
   id: "$Id: gdk_image.e,v 1.4 1999/01/18 19:54:55 pcj Exp $"
   
	 -- modified by Miron Brezuleanu, 99/03/17 ; made it useful
	 -- (it's no longer a placeholder)

class GDK_IMAGE
inherit 
   GDK_OBJECT
   EXG_CONSTANTS
   GTK_CONSTANTS
creation
   make_default,
   exg_create,
   make,
   make_from_window,
   make_from_pixmap,
   make_from_bitmap,
   make_with_defaults
   
feature -- Creation
   
   make_with_defaults(w, h: INTEGER) is
	 -- a wrapper for make, with some decent values
      do
	 make(gdk_image_fastest, system_visual, w, h)
      end
   
   make(type: GDK_IMAGE_TYPE; vis: GDK_VISUAL; w, h: INTEGER) is
	 -- makes a new image in memory
      do
	 gdkobject := ext_image_new(type.value, vis.to_external, w, h)
	 width := w
	 height := h
      end
   
   make_from_window(window: GDK_WINDOW; x, y, w, h: INTEGER) is
	 -- makes a new image from the data contained in a window
      require
	 window /= Void
      do
	 gdkobject := ext_image_get(window.to_external, x, y, w, h)
	 width := w
	 height := h
      end
   
   make_from_pixmap(pixmap: GDK_PIXMAP; x, y, w, h: INTEGER) is
	 -- GdkPixmap is a GdkWindow, so this does the same as 
	 -- `make_from_window', but using a pixmap
      require
	 pixmap /= Void
      do
	 gdkobject := ext_image_get(pixmap.to_external, x, y, w, h)
	 width := w
	 height := h
      end
   
   make_from_bitmap(bitmap: GDK_BITMAP; x, y, w, h: INTEGER) is
	 -- this should also work with a bitmap
      require
	 bitmap /= Void
      do
	 gdkobject := ext_image_get(bitmap.to_external, x, y, w, h)
	 width := w
	 height := h
      end
   
   sync is
	 -- this is very important; call this if your program
	 -- crashes!; If it doesn't stop crashing, then this wasn't
	 -- it!
      do
	 ext_flush
      end
   
   destroy is
	 -- deletes the image
      do
	 ext_image_destroy(gdkobject)
      end
   
feature -- access, but quite slow
   
   put_pixel, put_pixel_as_color(x, y: INTEGER; color: GDK_COLOR) is
	 -- guess...
      require
	 x >= 0
	 y >= 0
	 x < width
	 y < height
      do
	 ext_image_put_pixel(gdkobject, x, y, color.pixel)
      end
   
   put_pixel_as_integer(x, y, pixel_color: INTEGER) is
	 -- will use directly a pixel value, not a color from which 
	 -- to extract the pixel value; useful for those monocrome
	 -- images (like masks)
      require
	 x >= 0
	 y >= 0
	 x < width
	 y < height
      do
	 ext_image_put_pixel(gdkobject, x, y, pixel_color)
      end
   
feature -- Queries

   -- there are more variants to this, to make it more versatile
   
   pixel_as_color, pixel(x, y: INTEGER): GDK_COLOR is
	 -- this returns the color of (x,y) wrapped as a GDK_COLOR
      require
	 x >= 0
	 y >= 0
	 x < width
	 y < height
      do
	 !!Result.make_from_pixel(ext_image_get_pixel(gdkobject, x, y))
      end

   pixel_as_integer(x, y: INTEGER): INTEGER is
	 -- this returns the color of (x,y) as an integer (index or 
	 -- rgb triplet)
      require
	 x >= 0
	 y >= 0
	 x < width
	 y < height
      do
	 Result := ext_image_get_pixel(gdkobject, x, y)
      end
   
   width, height: INTEGER
   
feature { NONE }
   
   ext_image_new(t: INTEGER; v: POINTER; w, h: INTEGER): POINTER is
      external "C"
      alias "gdk_image_new"
      end
   
   ext_image_get(p: POINTER; x, y, w, h: INTEGER): POINTER is
      external "C"
      alias "gdk_image_get"
      end
   
   ext_image_destroy(p: POINTER) is
      external "C"
      alias "gdk_image_destroy"
      end
   
   ext_image_put_pixel(p: POINTER; x, y, pixel_color: INTEGER) is
      external "C"
      alias "gdk_image_put_pixel"
      end   
   
   ext_image_get_pixel(p: POINTER; x, y: INTEGER): INTEGER is
      external "C"
      alias "gdk_image_get_pixel"
      end   
   
   ext_flush is
      external "C"
      alias "gdk_flush"
      end
   
end
