indexing
   description: "gdk rectangle"
   status: "You may be able to redistribute and/or modify this program.% 
          % See the README for details."
   copyright: "Copyright (C) 1999 Paul C. Janzen"
   author: "Paul C. Janzen"
   id: "$Id: gdk_rectangle.e,v 1.5 1999/01/18 19:54:55 pcj Exp $"

	 -- modified by Miron Brezuleanu, on 1999/03/15

class GDK_RECTANGLE
inherit 
   GDK_OBJECT
creation
   make_default, 
   exg_create,
   make,
   intersect,
   union
   
feature -- Creation

   make(px, py, pwidth, pheight: INTEGER) is
	 -- creator, in GC-able space
      do
	 !!storage.make(ext_sizeof_gdk_rectangle)
	 gdkobject := storage.to_external
	 ext_new_rectangle(gdkobject, px, py, pwidth, pheight)
      end

   intersect(s1, s2: like Current) is
	 -- the current rectangle is the intersection of the two. The 
	 -- new rectangle is created in GC-able space
      do
	 !!storage.make(ext_sizeof_gdk_rectangle)
	 gdkobject := storage.to_external
	 ext_rectangle_intersect(s1.to_external, s2.to_external, gdkobject)
      end

   union(s1, s2: like Current) is
	 -- the current rectangle is the union of the two. The 
	 -- new rectangle is created in GC-able space
      do
	 !!storage.make(ext_sizeof_gdk_rectangle)
	 gdkobject := storage.to_external
	 ext_rectangle_union(s1.to_external, s2.to_external, gdkobject)
      end

   inside(px, py: INTEGER): BOOLEAN is
	 -- checks whether the given point is inside this rectangle
      do
	 if px >= x and px < x + width
	    and py >= y and py < y + height then
	    Result := True
	 end
      end
   
feature { NONE } -- storage
   
   storage: FIXED_ARRAY[CHARACTER]

feature -- Queries
   
   x: INTEGER is
	 -- left
      do
	 Result := ext_rectangle_x(gdkobject)
      end

   y: INTEGER is
	 -- top
      do
	 Result := ext_rectangle_y(gdkobject)
      end

   width: INTEGER is
	 -- width
      do
	 Result := ext_rectangle_width(gdkobject)
      end

   height: INTEGER is
	 -- height
      do
	 Result := ext_rectangle_height(gdkobject)
      end

feature {NONE} -- Private

   ext_sizeof_gdk_rectangle: INTEGER is
      external "C"
      end

   ext_new_rectangle(p: POINTER; px, py, pw, ph: INTEGER) is
      external "C"
      end

   ext_rectangle_x(p: POINTER): INTEGER is
      external "C"
      end

   ext_rectangle_y(p: POINTER): INTEGER is
      external "C"
      end

   ext_rectangle_width(p: POINTER): INTEGER is
      external "C"
      end

   ext_rectangle_height(p: POINTER): INTEGER is
      external "C"
      end

   ext_rectangle_intersect(s1, s2, p: POINTER) is
      external "C"
      alias "gdk_rectangle_intersect"
      end

   ext_rectangle_union(s1, s2, p: POINTER) is
      external "C"
      alias "gdk_rectangle_union"
      end

invariant
end 

