
indexing
   description: "glist"
   status: "You may be able to redistribute and/or modify this program.% 
          % See the README for details."
   copyright: "Copyright (C) 1999 Paul C. Janzen"
   author: "Paul C. Janzen and Francois Pennaneach"
   id: "$Id: glist.e,v 1.9 1999/01/18 19:54:56 pcj Exp $"

deferred class GLIST[T]

inherit 
   GENERAL 
      redefine 
	 is_equal 
      end
   MEMORY

feature -- Creation
   make is
      -- Creation and initialization of an empty list
      do 
	 gs_object := default_pointer
      end -- make
   
   from_collection(collection: COLLECTION[T]) is
      local
	 i: INTEGER
	 old_p, p: POINTER
      do
	 from 
	    i := collection.lower
	    !!fool_gc.make(1, 0)
	 until 
	    i > collection.upper
	 loop
	    !!stupid_storage.make(ext_glist_size)
	    p := stupid_storage.to_external
	    fool_gc.add_last(p)
	    if i = collection.lower then
	       gs_object := p
	    end
	    ext_glist_set_data(p, ext_rep(collection.item(i)))
	    ext_glist_set_prev(p, old_p)
	    if old_p /= default_pointer then
	       ext_glist_set_next(old_p, p)
	    end
	    old_p := p
	    i := i+1
	 end
      end
   
   exg_create ( p : POINTER ) is 
      do
	 gs_object := p 
      end
   
feature {ANY} -- Queries

   to_external : POINTER is
      do
	 Result := gs_object
      end

   to_array : ARRAY[T] is
      local 
	 i : INTEGER
      do
	 !!Result.make(lower, upper)
	 from i := lower 
	 until i > upper 
	 loop
	    Result.put(item(i), i)
	    i := i+1
	 end
      end

feature {ANY} -- Commands
   write is
	 -- for debugging purposes
      local
	 i : INTEGER
      do
	 from i:= lower until i > upper
	 loop
	    print(i);print("  ");print(item(i));print("%N")
	    i := i + 1
	 end -- loop
      end -- write

   count : INTEGER is
      do
	 Result := ext_length(gs_object)
      end -- count

   frozen lower : INTEGER is
      once
	 Result := 1
      end -- lower

   upper : INTEGER is
      do
	 Result := count
      end -- do

   is_equal(other : like Current) : BOOLEAN is
      do
	 Result := gs_object = other.to_external
      end -- is_equal

   append(element : T) is
      do
	 crash
	 gs_object := ext_append(gs_object, ext_rep(element))
      end -- append

   insert(element : T; position : INTEGER) is
      do
	 crash
	 gs_object := ext_insert(gs_object, ext_rep(element), position - 1)
      end -- insert

   prepend(element : T) is
      do
	 crash
	 gs_object := ext_prepend(gs_object, ext_rep(element))
      end -- add_first

   item(index : INTEGER) : T is
      do
	 Result := eiffel_rep ( ext_item ( gs_object, index - 1 ) )
      end -- item
   
feature {NONE} -- Private
   
   stupid_storage: FIXED_ARRAY[CHARACTER]

   gs_object : POINTER
	 -- the pointer to the object
   
   fool_gc: ARRAY[POINTER]
	 -- a lame trick to prevent the GC from collecting the 
	 -- allocated areas before collecting this GLIST

   -- Redefine this if you want the in-glist representation to be 
   -- something other than the pointer to the eiffel runtime struct

   ext_rep(object : T) : POINTER is
      do
	 Result := object.to_pointer
      end

   eiffel_rep ( p : POINTER ) : T is
      deferred
      end

   ext_alloc : POINTER is
      external "C"
      alias "g_list_alloc"
      end -- ext_alloc

   ext_append(c_object, element : POINTER) : POINTER is
      external "C"
      alias "g_list_append"
      end -- ext_append

   ext_insert(c_object, element : POINTER; position : INTEGER) : POINTER is
      external "C"
      alias "g_list_insert"
      end -- ext_insert

   ext_item(c_object : POINTER; index : INTEGER) : POINTER is
      external "C"
      alias "g_list_nth_data"
      end -- ext_item

   ext_length(c_object : POINTER) : INTEGER is
      external "C"
      alias "g_list_length"
      end -- ext_length

   ext_prepend(c_object, element : POINTER) : POINTER is
      external "C"
      alias "g_list_prepend"
      end -- ext_prepend

   ext_dispose( c_object : POINTER ) is
      external "C"
      alias "g_list_free"
      end
   
   ext_glist_size: INTEGER is
      external "C"
      end
   
   ext_glist_set_data(p1, p2: POINTER) is
      external "C"
      end
   
   ext_glist_set_prev(p1, p2: POINTER) is
      external "C"
      end
   
   ext_glist_set_next(p1, p2: POINTER) is
      external "C"
      end
   
invariant
end -- GLIST[T]

