indexing
   description: "gtk table"
   status: "You may be able to redistribute and/or modify this program.% 
          % See the README for details."
   copyright: "Copyright (C) 1999 Paul C. Janzen"
   author: "Paul C. Janzen and Francois Pennaneach"
   id: "$Id: gtk_table.e,v 1.8 1999/01/18 19:54:59 pcj Exp $"

class GTK_TABLE
inherit
   GTK_CONTAINER
   GTK_CONSTANTS
   GTK_FLAGS_NONE
creation
   exg_create,
   make,
   make_table
feature -- Creation
   make, make_table(rows, columns : INTEGER; homogeneous : BOOLEAN) is
      -- Creation and initialization
      do
	 gtkobject := ext_table_new(rows, columns, homogeneous)
	 attach_fill_x := True
	 attach_shrink_x := False
	 attach_expand_x := True
	 attach_fill_y := True
	 attach_shrink_y := False
	 attach_expand_y := True
	 build_options
	 padding_x := 0
	 padding_y := 0
      end -- make

feature {ANY} -- Queries
feature {ANY} -- Commands
   
   attach(child: GTK_WIDGET; 
	  left, right, top, bottom: INTEGER;
	  xopts, yopts: GTK_ATTACH_OPTIONS;
	  padx, pady: INTEGER) is
	 -- the classic `attach'
      do
	 ext_attach(gtkobject, child.to_external,
		    left, right, top, bottom,
		    xoptions.value, yoptions.value,
		    padding_x, padding_y)	 
      end
   
   attach2(child : GTK_WIDGET; left, right, top, bottom : INTEGER) is
	 -- the new&improved `attach': you can set options for
	 -- multiple widgets then use this call to attach the widgets
      do
	 ext_attach(gtkobject, child.to_external, 
		    left, right, top, bottom, 
		    xoptions.value, yoptions.value, 
		    padding_x, padding_y)
      end -- attach2

   attach_defaults(child : GTK_WIDGET; left, right, top, bottom : INTEGER) is
      do
	 ext_attach_defaults(gtkobject, child.to_external, 
			     left, right, top, bottom)
      end -- attach_defaults

   -- This method has no GTK counterpart.  Use it when you need the 
   -- child to continue to work properly afterwards. 
   detach ( child : GTK_WIDGET ) is
      do
	 child.ref
	 remove(child)
      end

   set_col_spacing(column, spacing : INTEGER) is
      do
	 ext_set_col_spacing(gtkobject, column, spacing)
      end -- set_col_spacing

   set_col_spacings(spacing : INTEGER) is
      do
	 ext_set_col_spacings(gtkobject, spacing)
      end -- set_col_spacings

   set_row_spacing(row, spacing : INTEGER) is
      do
	 ext_set_row_spacing(gtkobject, row, spacing)
      end -- set_row_spacing

   set_row_spacings(spacing : INTEGER) is
      do
	 ext_set_row_spacings(gtkobject, spacing)
      end -- set_row_spacings

   set_homogeneous(homogeneous : BOOLEAN) is
      do
	 ext_set_homogeneous ( gtkobject, homogeneous ) 
      end
   
   set_attach_fill_x(nv: BOOLEAN) is
      do
	 attach_fill_x := nv
	 build_options
      end
   
   set_attach_shrink_x(nv: BOOLEAN) is
      do
	 attach_shrink_x := nv
	 build_options
      end
   
   set_attach_expand_x(nv: BOOLEAN) is
      do
	 attach_expand_x := nv
	 build_options
      end
   
   set_attach_fill_y(nv: BOOLEAN) is
      do
	 attach_fill_y := nv
	 build_options
      end
   
   set_attach_shrink_y(nv: BOOLEAN) is
      do
	 attach_shrink_y := nv
	 build_options
      end
   
   set_attach_expand_y(nv: BOOLEAN) is
      do
	 attach_expand_y := nv
	 build_options
      end
   
   set_padding_x(nv: INTEGER) is
      do
	 padding_x := nv
	 build_options
      end
   
   set_padding_y(nv: INTEGER) is
      do
	 padding_y := nv
	 build_options
      end
   
   build_options is
	 -- turns the boolean values in values usable by `attach'
      do
	 xoptions := gtk_attach_none
	 if attach_fill_x then
	    xoptions := xoptions and gtk_fill
	 end
	 if attach_shrink_x then
	    xoptions := xoptions and gtk_shrink
	 end
	 if attach_expand_x then
	    xoptions := xoptions and gtk_expand
	 end
	 yoptions := gtk_attach_none
	 if attach_fill_y then
	    yoptions := yoptions and gtk_fill
	 end
	 if attach_shrink_y then
	    yoptions := yoptions and gtk_shrink
	 end
	 if attach_expand_y then
	    yoptions := yoptions and gtk_expand
	 end
      end
   
feature -- some useful options for attaching
   
   attach_fill_x, attach_shrink_x, attach_expand_x: BOOLEAN
   attach_fill_y, attach_shrink_y, attach_expand_y: BOOLEAN
   xoptions, yoptions: GTK_ATTACH_OPTIONS
   padding_x, padding_y: INTEGER

feature {NONE} -- Private
   ext_table_new(rows, columns : INTEGER; homogeneous : BOOLEAN) : POINTER is
      external "C"
      alias "gtk_table_new"
      end -- ext_table_new

   ext_attach(c_object, child : POINTER; 
	      left, right, top, bottom, 
	      xoptions_, yoptions_, xpadding, ypadding : INTEGER) is
      external "C"
      alias "gtk_table_attach"
      end -- ext_attach

   ext_attach_defaults(c_object, child : POINTER; 
		       left, right, top, bottom : INTEGER) is
      external "C"
      alias "gtk_table_attach_defaults"
      end -- ext_attach

   ext_set_col_spacing(c_object : POINTER; column, spacing : INTEGER) is
      external "C"
      alias "gtk_table_set_col_spacing"
      end -- ext_set_col_spacing

   ext_set_col_spacings(c_object : POINTER; spacing : INTEGER) is
      external "C"
      alias "gtk_table_set_col_spacings"
      end -- ext_set_col_spacing

   ext_set_row_spacing(c_object : POINTER; row, spacing : INTEGER) is
      external "C"
      alias "gtk_table_set_row_spacing"
      end -- ext_set_row_spacing

   ext_set_row_spacings(c_object : POINTER; spacing : INTEGER) is
      external "C"
      alias "gtk_table_set_row_spacings"
      end -- ext_set_row_spacing

   ext_set_homogeneous(table : POINTER ; homogeneous : BOOLEAN) is
      external "C"
      alias "gtk_table_set_homogeneous"
      end
   
invariant
end -- GTK_TABLE

