
indexing
   author: "Miron Brezuleanu"
   purpose: "hold the event-handling part of the widget class"

class GTK_WIDGET_COMMAND

inherit
   GTK_COMMAND_EVENT
   GTK_CONSTANTS

feature -- event handling

   Delete_event: INTEGER is unique
   Button_press_event, Button_release_event: INTEGER is unique
   Motion_notify_event: INTEGER is unique
   Expose_event: INTEGER is unique
   Key_press_event, Key_release_event: INTEGER is unique
   Enter_notify_event, Leave_notify_event: INTEGER is unique
   Configure_event: INTEGER is unique
   Focus_in_event, Focus_out_event: INTEGER is unique
   Map_event, Unmap_event: INTEGER is unique
   
   Delete_event_b: INTEGER is unique
   Button_press_event_b, Button_release_event_b: INTEGER is unique
   Motion_notify_event_b: INTEGER is unique
   Expose_event_b: INTEGER is unique
   Key_press_event_b, Key_release_event_b: INTEGER is unique
   Enter_notify_event_b, Leave_notify_event_b: INTEGER is unique
   Configure_event_b: INTEGER is unique
   Focus_in_event_b, Focus_out_event_b: INTEGER is unique
   Map_event_b, Unmap_event_b: INTEGER is unique

   activate_event_handling(w: GTK_WIDGET) is
	 -- command that starts event handling for all known events
      do
	 widget := w
	 w.set_events(gdk_exposure_mask
		    and gdk_pointer_motion_mask
		    and gdk_button_press_mask
		    and gdk_button_release_mask
		    and gdk_key_press_mask
		    and gdk_key_release_mask
		    and gdk_enter_notify_mask
		    and gdk_leave_notify_mask
		    and gdk_focus_change_mask
		    and gdk_structure_mask)
	 w.signal_connect_after("delete_event", Current, Delete_event)
	 w.signal_connect_after("button_press_event", 
			      Current, Button_press_event)
	 w.signal_connect_after("button_release_event", 
			      Current, Button_release_event)
	 w.signal_connect_after("motion_notify_event", 
			      Current, Motion_notify_event)
	 w.signal_connect_after("expose_event", Current, Expose_event)
	 w.signal_connect_after("key_press_event", Current, Key_press_event)
	 w.signal_connect_after("key_release_event", Current,
				Key_release_event)
	 w.signal_connect_after("enter_notify_event", 
			      Current, Enter_notify_event)
	 w.signal_connect_after("leave_notify_event", 
			      Current, Leave_notify_event)
	 w.signal_connect_after("configure_event", Current, Configure_event)
	 w.signal_connect_after("focus_in_event", Current, Focus_in_event)
	 w.signal_connect_after("focus_out_event", Current, Focus_out_event)
	 w.signal_connect_after("map_event", Current, Map_event)
	 w.signal_connect_after("unmap_event", Current, Unmap_event)	 
	    
	 w.signal_connect("delete_event", Current, Delete_event_b)
	 w.signal_connect("button_press_event", Current, Button_press_event_b)
	 w.signal_connect("button_release_event", 
			Current, Button_release_event_b)
	 w.signal_connect("motion_notify_event", Current,
			  Motion_notify_event_b)
	 w.signal_connect("expose_event", Current, Expose_event_b)
	 w.signal_connect("key_press_event", Current, Key_press_event_b)
	 w.signal_connect("key_release_event", Current, Key_release_event_b)
	 w.signal_connect("enter_notify_event", Current, Enter_notify_event_b)
	 w.signal_connect("leave_notify_event", Current, Leave_notify_event_b)
	 w.signal_connect("configure_event", Current, Configure_event_b)
	 w.signal_connect("focus_in_event", Current, Focus_in_event_b)
	 w.signal_connect("focus_out_event", Current, Focus_out_event_b)
	 w.signal_connect("map_event", Current, Map_event_b)
	 w.signal_connect("unmap_event", Current, Unmap_event_b)	 
      end

   execute(event: GDK_EVENT; data: ANY): BOOLEAN is
	 -- event handler for the current widget
      local
	 type: INTEGER_REF
	 b_event: GDK_EVENT_BUTTON
	 m_event: GDK_EVENT_MOTION
	 k_event: GDK_EVENT_KEY
	 e_event: GDK_EVENT_EXPOSE
	 c_event: GDK_EVENT_CONFIGURE
	 w: GTK_WIDGET
      do
	 type ?= data
	 w := widget
	 w.set_event_result(False)
	 inspect 
	    type.item
	 when Delete_event then
	    w.delete_handler
	 when Button_press_event then
	    b_event ?= event
	    w.button_press_handler(b_event)
	 when Button_release_event then
	    b_event ?= event
	    w.button_release_handler(b_event)
	 when Motion_notify_event then
	    m_event ?= event
	    w.motion_notify_handler(m_event)
	 when Expose_event then
	    e_event ?= event
	    w.expose_handler(e_event.area)
	 when Key_press_event then
	    k_event ?= event
	    w.key_press_handler(k_event)
	 when Key_release_event then
	    k_event ?= event
	    w.key_release_handler(k_event)
	 when Enter_notify_event then
	    w.enter_notify_handler
	 when Leave_notify_event then
	    w.leave_notify_handler
	 when Configure_event then
	    c_event ?= event	    
	    w.configure_handler(c_event.area)
	 when Focus_in_event then
	    w.focus_in_handler
	 when Focus_out_event then
	    w.focus_out_handler
	 when Map_event then
	    w.map_handler
	 when Unmap_event then
	    w.unmap_handler
	 when Delete_event_b then
	    w.delete_handler_b
	 when Button_press_event_b then
	    b_event ?= event
	    w.button_press_handler_b(b_event)
	 when Button_release_event_b then
	    b_event ?= event
	    w.button_release_handler_b(b_event)
	 when Motion_notify_event_b then
	    m_event ?= event
	    w.motion_notify_handler_b(m_event)
	 when Expose_event_b then
	    e_event ?= event
	    w.expose_handler_b(e_event.area)
	 when Key_press_event_b then
	    k_event ?= event
	    w.key_press_handler_b(k_event)
	 when Key_release_event_b then
	    k_event ?= event
	    w.key_release_handler_b(k_event)
	 when Enter_notify_event_b then
	    w.enter_notify_handler_b
	 when Leave_notify_event_b then
	    w.leave_notify_handler_b
	 when Configure_event_b then
	    c_event ?= event	    
	    w.configure_handler_b(c_event.area)
	 when Focus_in_event_b then
	    w.focus_in_handler_b
	 when Focus_out_event_b then
	    w.focus_out_handler_b
	 when Map_event_b then
	    w.map_handler_b
	 when Unmap_event_b then
	    w.unmap_handler_b
	 end
	 Result := w.event_result
      end

   widget: GTK_WIDGET
	 -- this is the associated widget

end
      
