
indexing
   purpose: "define the widget that provides a modified%
   %rubber band selection"
   author: "Miron Brezuleanu"
   
class RS_CANVAS

inherit 
   GTK_DRAWING_AREA
      redefine
	 make,
	 expose_handler,
	 button_1_press_handler,
	 button_1_release_handler,
	 button_2_release_handler,
	 button_3_release_handler,
	 motion_notify_handler,
	 key_release_handler
      end
   EXG_CONSTANTS
   GTK_CONSTANTS
   
creation
   make
   
feature
   
   make is
	 -- sets things up
      do
	 Precursor
	 set_size (200, 200)
	 activate_event_handling
	 set_flags (gtk_can_focus)
	 selection := Void
	 step := 0
	 dragging := False
      end
   
   draw_background is
	 -- if you want to use this class in your programs, you can 
	 -- just inherit it and redefine this command, which draws 
	 -- the background under the selection
      
	 -- now it draws just a white background
      local
	 gc: GDK_GC
      do
	 !! gc.make (Current)
	 gc.set_foreground (exg_white)
	 draw_rectangle (gc, True, 
			 0, 0, 
			 allocation.width, allocation.height)
	 gc.destroy
      end
   
   zoom is
	 -- uses the selection to whatever purpose ...
      require
	 selection /= Void
      local
	 emb: EXG_MESSAGE_BOX
      do
	 !! emb.make
	 emb.set_title ("Zoom!")
	 emb.set_message (selection.to_string)
	 emb.run
      end
   
   reset is
	 -- prepares the widget for a new selection
      do
	 selection := Void
	 step := 0
      end
   
   expose_handler (area: GDK_RECTANGLE) is
	 -- redraws the canvas
      local
	 gc: GDK_GC
	 r: GDK_RECTANGLE
      do
	 draw_background
	 !! gc.make (Current)
	 inspect step
	 when 0 then
	 when 1 then
	    gc.set_function (gdk_xor)
	    gc.set_foreground (exg_white) 
	    -- `exg_white' is essential for the xor stuff to work!!
	    r := rectangle_from_rubberband
	    draw_rectangle (gc, False,
			    r.x, r.y, r.width, r.height)
	 when 2 then
	    selection.draw (Current)
	 end
	 gc.destroy
      end
   
   button_1_press_handler (eb: GDK_EVENT_BUTTON) is
	 -- handles the creation & modification of the selection
      do
	 expect_key_press := False
	 inspect step
	 when 0 then
	    dragging := True
	    !! p1.make (eb.int_x, eb.int_y)
	    !! p2.make (eb.int_x, eb.int_y)
	    step := 1
	    expose_handler (Void)
	 when 1 then
	 when 2 then
	    !! p1.make (eb.int_x, eb.int_y)
	    if not selection.matches_point (p1) then 
	       -- the old `selection restart' code
	       step := 0
	       button_1_press_handler (eb) -- nasty kludge
	       
	       selection := Void
	       -- _plus_ this line
	    else
	       dragging := True
	    end
	 end
      end
   
   button_1_release_handler (eb: GDK_EVENT_BUTTON) is
	 -- handles the creation & modification of the selection
      do
	 expect_key_press := False
	 inspect step
	 when 0 then
	 when 1 then
	    dragging := False
	    step := 2
	    !! selection.make (rectangle_from_rubberband)
	    expose_handler (Void)
	 when 2 then
	    if dragging then
	       dragging := False
	       expose_handler (Void)
	    end
	 end
      end
   
   button_2_release_handler (eb: GDK_EVENT_BUTTON) is
	 -- displays the current selection (if there is one)
      local
	 aux: GDK_POINT
      do
	 expect_key_press := False
	 inspect step
	 when 0 then
	 when 1 then
	 when 2 then
	    !! aux.make (eb.int_x, eb.int_y)
	    if selection.matches_point (aux) then
	       expect_key_press := True
	    end
	 end
      end
   
   button_3_release_handler (eb: GDK_EVENT_BUTTON) is
	 -- displays the current selection (if there is one)
      do
	 expect_key_press := False
	 inspect step
	 when 0 then
	 when 1 then
	 when 2 then
	    zoom
	    reset
	 end
      end
   
   motion_notify_handler (em: GDK_EVENT_MOTION) is
	 -- handles the creation & modification of the selection
      local
	 aux: GDK_POINT
      do
	 inspect step
	 when 0 then
	 when 1 then
	    if dragging then
	       !! p2.make (em.int_x, em.int_y)
	       expose_handler (Void)
	    end
	 when 2 then
	    if dragging then
	       !! aux.make (em.int_x, em.int_y)
	       selection.points.put (aux, selection.matched_point_index)
	       expose_handler (Void)
	    end
	 end
      end
   
   key_release_handler (ek: GDK_EVENT_KEY) is
	 -- handles the creation & modification of the selection
      local
	 aux: CHARACTER
      do
	 inspect step
	 when 0 then
	 when 1 then
	 when 2 then
	    if expect_key_press 
	       and ek.keyval >= ('1').to_integer
	       and ek.keyval <= ('6').to_integer then
	       aux := ek.keyval.to_character
	       inspect aux
	       when '1'..'4' then
		  selection.set_index (ek.keyval - ('0').to_integer,
				       selection.matched_point)
	       when '5' then
		  selection.rotate_left
	       when '6' then
		  selection.rotate_right
	       end
	       expose_handler (Void)
	    end
	 end
      end
   
feature -- queries & attributes
   
   rectangle_from_rubberband: GDK_RECTANGLE is
	 -- will use p1, p2 to give a rectangle as we usually expect 
	 -- one to be (location of top corner, width, height)
      local
	 left, top, width, height: INTEGER
      do
	 left := p1.x.min (p2.x)
	 top := p1.y.min (p2.y)
	 width := p1.x.max (p2.x) - left + 1
	 height := p1.y.max (p2.y) - top + 1
	 !! Result.make (left, top, width, height)
      end
   
   selection: RUBBER_BAND_SELECTION
	 -- the selection object
   
   step: INTEGER
	 -- if 0: we don't have a selection; if 1: we have a normal
	 -- rubber band selection (dragging phase); if 2: we're
	 -- modifying the position of individual points
   
   dragging: BOOLEAN
	 -- `True' if we are dragging something
   
   p1, p2: GDK_POINT
	 -- the corners used during `step=1'   
   
   expect_key_press: BOOLEAN
	 -- the last mouse click was with button 2 and we should 
	 -- expect the next character to try and change the order of 
	 -- the points in the selection
   
end
