
indexing
   purpose: "modelates a selection object"
   
class RUBBER_BAND_SELECTION
   
inherit
   SINGLETON_MAKER
   GTK_CONSTANTS
   EXG_CONSTANTS
   
creation
   make
   
feature
   
   make (r: GDK_RECTANGLE) is
	 -- builds the selection object from the given rectangle
      local
	 p: GDK_POINT
      do
	 !ARRAY[GDK_POINT]! points.make (1, 0)
	 !! p.make (r.x, r.y + r.height - 1)
	 points.add_last (p)
	 !! p.make (r.x, r.y)
	 points.add_last (p)
	 !! p.make (r.x + r.width - 1, r.y)
	 points.add_last (p)
	 !! p.make (r.x + r.width - 1,
		    r.y + r.height - 1)
	 points.add_last (p)
      end
   
   draw (d: GDK_DRAWABLE_WRAPPER) is
	 -- draws the visual representation of the selection on the
	 -- given drawable
      local
	 gc: GDK_GC
	 i : INTEGER
	 cp, cp2: GDK_POINT
	 size: INTEGER
	 arrow: EXG_ARROW
      do
	 !! gc.make (d)
	 gc.set_foreground (exg_white)
	 gc.set_function (gdk_xor)
	 from i := 1 until i > points.upper loop
	    cp := points.item (i)
	    if i = 1 then	     
	       size := Large_point_square_size
	       gc.set_line_width (2)
	    else
	       size := Point_square_size
	       gc.set_line_width (1)
	    end
	    d.draw_rectangle (gc, False,
			      cp.x - size, cp.y - size,
			      size * 2 + 1, size * 2 + 1)		      
	    i := i + 1
	 end
	 from i := 1 until i > points.upper-1 loop
	    cp := points.item (i)
	    cp2 := points.item (i + 1)
	    d.draw_line (gc, cp.x, cp.y, cp2.x, cp2.y)
	    i := i + 1
	 end
	 cp := points.item (4)
	 cp2 := points.item (1)
	 d.draw_line (gc, cp.x, cp.y, cp2.x, cp2.y)
	 
	 cp := points.item (1)
	 cp2 := points.item (2)
	 !! arrow.make (d)
	 arrow.set_head_distance (0.25)
	 arrow.set_head_wing (8)
	 arrow.draw (cp.x, cp.y, cp2.x, cp2.y)
	 
	 gc.destroy
      end
   
   to_string: STRING is
	 -- provides a text representation of the object
      local
	 i: INTEGER
      do
	 Result := clone ("Selection:%N")
	 from i := 1 until i > points.upper loop
	    Result.append("Point " + i.to_string + " ")
	    Result.append(points.item (i).x.to_string + ",")
	    Result.append(points.item (i).y.to_string + "%N")
	    i := i + 1
	 end
      end
   
   set_index (idx: INTEGER; pt: GDK_POINT) is
	 -- plays with the points to make `pt' have the index `idx'
      require
	 points.has (pt)
      local
	 actual_idx: INTEGER
      do
	 actual_idx := points.index_of (pt)
	 if actual_idx /= idx then
	    points.swap (actual_idx, idx)
	 end
      end
   
   rotate_left is
	 -- 2->1, 3->2, 4->3, 1->4
      local
	 i: INTEGER
	 aux: GDK_POINT
      do
	 aux := points.item (1)
	 from i := 1 until i > points.upper - 1 loop
	    points.put (points.item(i + 1), i)
	    i := i + 1
	 end
	 points.put (aux, points.upper)
      end
   
   rotate_right is
	 -- 1->2, 2->3, 3->4, 4->1
      local
	 i: INTEGER
	 aux: GDK_POINT
      do
	 aux := points.item (points.upper)
	 from i := points.upper until i < points.lower + 1 loop
	    points.put (points.item(i-1), i)
	    i := i - 1
	 end
	 points.put (aux, points.lower)
      end
   
feature -- queries & attributes
   
   matches_point (p: GDK_POINT): BOOLEAN is
	 -- checks if `p' is close to any of the `points' of the
	 -- selection'; if so, returns True and `matched_point' holds
	 -- the point that matched and `matched_point_index' its index
      local
	 i: INTEGER
	 size: INTEGER
      do
	 from i := 1 until i > points.upper loop
	    if i = 1 then	     
	       size := Large_point_square_size
	    else
	       size := Point_square_size
	    end
	    if (p.x - points.item (i).x).abs < size
	       and (p.y - points.item (i).y).abs < size then
	       Result := True
	       matched_point := points.item (i)
	       matched_point_index := i
	       i := points.upper
	    end
	    i := i + 1
	 end
      end
	 
   matched_point: GDK_POINT
   matched_point_index: INTEGER
   
   points: COLLECTION[GDK_POINT]
   
end
	 
