
indexing
   author: "Miron Brezuleanu"
   purpose: "provide a fast way to query the user for a color"
   
class EXG_COLOR_SELECTION_MODAL
   
inherit   
   GTK_MAIN
   GTK_COMMAND
   EXG_CONSTANTS
   
creation
   make
   
feature
   
   make is 
	 -- will initialize the fields
      do
	 answer := clone("cancel")
	 title := clone("")
      end
   
   set_opacity_value(nv: DOUBLE) is
	 -- setter command for `opacity_value'
      require
	 nv >= 0.0
	 nv <= 1.0
      do
	 opacity_value := nv
      end
   
   set_color(ncolor: GDK_COLOR) is
	 -- will set an initial value for the color; call this
	 -- before `run'
      require
	 ncolor /= Void
      do
	 !!color_as_array.make(1, 4)	 
	 color_as_array.put(ncolor.red.to_double / 65536, 1)
	 color_as_array.put(ncolor.green.to_double / 65536, 2)
	 color_as_array.put(ncolor.blue.to_double / 65536, 3)
      end
   
   set_opacity(nv: BOOLEAN) is
	 -- setter command for `opacity'
      do
	 opacity := nv
      end
   
   set_title(nv: STRING) is
	 -- will set an initial value for the title; call this before
	 -- `run'
      require
	 nv /= Void
      do
	 title := nv
      end
   
   run is
	 -- the main routine; will show the modal dialog and wait for 
	 -- the user to select a color or cancel the whole thing
      local	 
      do
	 !!colorsel.make_dialog(title)
	 colorsel.colorsel.set_opacity(opacity)
	 if opacity then
	    color_as_array.put(opacity_value, 4)
	 end
	 if color_as_array /= Void then
	    colorsel.colorsel.set_color(color_as_array)
	 end
	 colorsel.ok_button.signal_connect("clicked", Current, "ok")
	 colorsel.cancel_button.signal_connect("clicked", Current,
					       "cancel")
	 colorsel.set_modal(True)
	 colorsel.show
	 dialog_stack.push(colorsel)
	 from
	    can_exit := False
	 until
	    can_exit
	 loop
	    yield
	 end
	 dialog_stack.pop
      end
   
   execute(d: ANY) is
	 -- will answer to button presses
      local
	 s: STRING
	 fa_color: FIXED_ARRAY[DOUBLE]
      do
	 s ?= d
	 check
	    s /= Void
	 end
	 if equal(s, "ok") then
	    fa_color := colorsel.colorsel.color
	    if opacity then
	       opacity_value := fa_color.item(3)
	    end
	    !!color.make((fa_color.item(0) * 65535).rounded,
			 (fa_color.item(1) * 65535).rounded,
			 (fa_color.item(2) * 65535).rounded)
	    answer := "ok"
	    can_exit := True
	    colorsel.destroy
	 elseif equal(s, "cancel") then
	    answer := "cancel"
	    can_exit := True
	    colorsel.destroy
	 elseif equal(s, "destroy") and not can_exit then
	    answer := "cancel"
	    can_exit := True
	    colorsel.destroy
	 end
      end
   
feature -- attributes & queries
   
   title: STRING
	 -- the title of the dialog
   
   answer: STRING
	 -- the result: "ok" if the user pushed the "ok" button and 
	 -- so on
   
   color_as_array: ARRAY[DOUBLE]
   opacity: BOOLEAN
   can_exit: BOOLEAN
   color: GDK_COLOR
	 -- the requested color (filled in by `run')
   
   opacity_value: DOUBLE
	 -- the opacity value of the requested color (meaningful only 
	 -- if the user set `opacity' to `True')
   
   colorsel: GTK_COLOR_SELECTION_DIALOG
	 -- internal stuff
   
end
   
