
indexing
   author: "Miron Brezuleanu"
   purpose: "This class is meant to provide a template for dialog-box building;%
   % for a minimal use one should redefine `build_vbox', `commit'%
   % and leave the rest as it is; for a more involved use,%
   % of course you can redefine as much as you want! (when redefining %
   %commit don't forget about setting `commit_ok' to True if %
   %everythings OK - otherwise the dialog won't close!)"
	    
deferred class EXG_DIALOG
   
inherit
   GTK_COMMAND
   GTK_MAIN
   GTK_CONSTANTS
   EXG_CONSTANTS
   
feature -- comands
   
   make is
	 -- will initialize the dialog
      do
	 answer := clone("cancel")
	 commit_ok := False
	 can_exit := False
	 window := Void
	 vbox := Void
      end
   
   run is
	 -- will run the dialog; won't return until the dialog is
	 -- closed; 
      do
	 build_window
	 window.show
	 dialog_stack.push(window)
	 from
	    can_exit := False
	 until
	    can_exit
	 loop
	    yield
	 end	 
	 dialog_stack.pop
      end
   
   build_vbox is
	 -- this is one of the functions you redefine; here, in the 
	 -- vbox, you can add your own controls to receive input; 
	 -- don't show the vbox; this will be done by the dialog; 
	 -- this command is also an opportunity to call `set_title';
      deferred
      end
   
   commit is
	 -- this is called whenever the "ok" button (or the "apply" 
	 -- button) is clicked; here you can read the data from your 
	 -- controls; if you're satisfied with the input, set 
	 -- `commit_ok' to `True'; otherwise set it to `False' to 
	 -- prevent the "ok" button from closing the dialog
      deferred
      end
   
   set_title(title: STRING) is
	 -- will set the title of the dialog
      do
	 window.set_title(title)
      end
   
feature { NONE } -- private helper subroutines
   
   execute(d: ANY) is
	 -- will handle the commands from the buttons
      local
	 s: STRING
      do
	 s ?= d
	 check
	    s /= Void
	 end
	 if equal(s, "ok") then
	    commit
	    if commit_ok then
	       answer := clone("ok")
	       quit_dialog
	    end
	 elseif equal(s, "apply") then
	    commit
	 elseif equal(s, "cancel") then
	    answer := clone("cancel")
	    quit_dialog
	 elseif equal(s, "destroy") and can_exit = False then
	    answer := clone("cancel")
	    quit_dialog
	 end
      end
   
   quit_dialog is
      do
	 can_exit := True
	 window.destroy
      end
   
   build_window is
      local
	 button: GTK_BUTTON
	 hbox: GTK_BOX
	 big_vbox: GTK_BOX
      do
	 !!window.make_with_type(gtk_window_dialog)
	 !GTK_VBOX!big_vbox.make(False, 0)
	 build_vbox
	 big_vbox.pack_start(vbox, True, True, 0)
	 vbox.show
	 !GTK_HBOX!hbox.make(False, 2)
	 !!button.make_with_label("Cancel")	 
	 button.signal_connect("clicked", Current, "cancel")
	 hbox.pack_end(button, False, False, 0)
	 button.show
	 !!button.make_with_label("Apply")	 
	 button.signal_connect("clicked", Current, "apply")
	 hbox.pack_end(button, False, False, 0)
	 button.show
	 !!button.make_with_label("Ok")	 
	 button.signal_connect("clicked", Current, "ok")
	 hbox.pack_end(button, False, False, 0)
	 button.show
	 big_vbox.pack_start(hbox, False, False, 0)
	 hbox.show
	 window.add(big_vbox)
	 big_vbox.show
	 window.signal_connect("destroy", Current, "destroy")
	 window.set_position(gtk_win_pos_center)
	 window.set_modal(True)
	 window.set_policy(False, False, False)
      end
   
feature { NONE } -- private attributes
   
   can_exit: BOOLEAN
	 -- if true, one can exit the local loop
   
   window: GTK_WINDOW    
	 -- this is the window of the dialog, containing the `vbox'
	 -- widget
   
feature -- public attributes   
   
   vbox: GTK_VBOX
	 -- this is the component you control; you can add here your 
	 -- personalized stuff
   
   answer: STRING 
	 -- what did the user press: "ok", "cancel"; "cancel" is also 
	 -- used if the user closes the window
   
   commit_ok: BOOLEAN
	 -- you set this variable in `commit'; if you don't like the 
	 -- input, you set it to `False'
end
