
indexing
   author: "Miron Brezuleanu"
   purpose: "main wrapper class for the imlib; I used the Imlib-devel docs to"
   "write this wrapper"
   
class IMLIB_IMAGE
   
creation
   make,
   make_from_pixmap,
   load
   
feature
   
   make, make_from_pixmap(ppixmap: GDK_PIXMAP; x, y, w, h: INTEGER) is
	 -- builds the imlib_image from the given pixmap
      require
	 ppixmap /= Void
      local
	 pmask: POINTER
      do
	 init
	 if ppixmap.mask /= Void then
	    pmask := ppixmap.mask.to_external
	 else
	    pmask := default_pointer
	 end
	 imlib_object := ext_create_image_from_drawable(ppixmap.to_external, 
							pmask,
							x, y, w, h)
	 check
	    imlib_object /= default_pointer
	 end
	 get_modifiers
	 get_shape
      ensure
	 imlib_object /= default_pointer
      end
   
   load(filename: STRING) is
	 -- loads an image; in Win95 this works only with jpeg, png and
	 -- gif files, so it will be restricted to these
      require
	 supports_extension(filename)
      do
	 init
	 imlib_object := ext_load_image(filename.to_external)
	 check
	    imlib_object /= default_pointer
	 end
	 get_modifiers
	 get_shape
      ensure
	 imlib_object /= default_pointer
      end
   
   render(w, h: INTEGER) is
	 -- will render the image to the specified size
      do
	 ext_render(imlib_object, w, h)
      end   
   
   apply(w: GDK_WINDOW) is
	 -- will `apply' this image to `w' (shape_combine_mask, etc.)
      do
	 ext_apply(imlib_object, w.to_external)
      end
   
   destroy is
	 -- will delete the image and place it on the cache
      do
	 ext_destroy_image(imlib_object)
      end
   
   flip_horizontal is
	 -- will flip the image horizontally; requires a re-render
      do
	 ext_flip_horizontal(imlib_object)
      end
   
   flip_vertical is
	 -- will flip the image vertically; requires a re-render
      do
	 ext_flip_vertical(imlib_object)
      end
   
   save(filename: STRING) is
	 -- will use the Imlib function for saving, which will
	 -- determine the file type automatically; however, because of
	 -- Win32 limitations, only a few filetypes are actually usable
	 -- under Win32 (JPEG, PNG, maybe GIF)
      require
	 supports_extension(filename)
      do
	 ext_save(imlib_object, filename.to_external, default_pointer)
      end
   
   set_shape(r, g, b: INTEGER) is
	 -- a function that sets the imlib_image's transparent color
      do
	 ext_set_image_shape(imlib_object, r, g, b)
	 shape_red := r
	 shape_green := g
	 shape_blue := b
      end
   
   set_modifiers(pgamma, pbrightness, pcontrast: DOUBLE) is
	 -- for this to become effective you need to re-render
      do
	 ext_set_image_modifiers(imlib_object, pgamma, pbrightness, pcontrast)
	 gamma := pgamma
	 brightness := pbrightness
	 contrast := pcontrast
      end
   
feature -- queries
   
   shape_red, shape_green, shape_blue: INTEGER
   
   gamma, brightness, contrast: DOUBLE
   
   to_external: POINTER is
      do
	 Result := imlib_object
      end
   
   width: INTEGER is
      do
	 Result := ext_rgb_width(imlib_object)	 
      end
   
   height: INTEGER is
      do
	 Result := ext_rgb_height(imlib_object)
      end
   
   pixmap: GDK_PIXMAP is
      local
	 i_pixmap: IMLIB_PIXMAP
      do
	 !!i_pixmap.make_with_imlib(Current)
	 Result := i_pixmap
      end
   
   supports_extension(filename: STRING): BOOLEAN is
	 -- tells us if we can handle this
      
	 -- well, right now this is disabled by the last command
      do
	 if filename.has_suffix(".jpg") or
	    filename.has_suffix(".jpeg") or
	    filename.has_suffix(".png") or
	    filename.has_suffix(".gif") then
	    Result := True
	 end
	 
	 Result := True
      end
   
feature { NONE } -- initialization
   
   init is
	 -- this must be called before any other Imlib operation
      
	 -- my solution is to call it as the first thing in `make', 
	 -- as the first thing one would do with Imlib is the loading 
	 -- of an image
      
	 -- this also forces the application to use the Imlib visual 
	 -- and colormap (which probably matters for <=8bit depths)
      once
	 ext_init
	 ext_push_visual(ext_get_visual)
	 ext_push_colormap(ext_get_colormap)
      end

feature { NONE } -- queries that are internal
   
   get_modifiers is
	 -- one may get the values through this
      do
	 ext_get_image_modifiers(imlib_object, $gamma, $brightness, $contrast)
      end
   
   get_shape is
	 -- a function that queries the imlib_image's transparent 
	 -- color, the result being retrievable through `shape_red', 
	 -- etc. queries
      do
	 ext_get_image_shape(imlib_object, 
			     $shape_red, $shape_green, $shape_blue)
      end
   
feature { NONE } -- external stuff
   
   imlib_object: POINTER
   
   ext_set_image_shape(p: POINTER; i1, i2, i3: INTEGER) is
      external "C"
      end
   
   ext_get_image_shape(p1, p2, p3, p4: POINTER) is
      external "C"
      end
   
   ext_set_image_modifiers(p: POINTER; i1, i2, i3: DOUBLE) is
      external "C"
      end
   
   ext_get_image_modifiers(p1, p2, p3, p4: POINTER) is
      external "C"
      end
   
   ext_create_image_from_drawable(p1, p2: POINTER; 
				  i1, i2, i3, i4: INTEGER): POINTER is      
      external "C"
      alias "gdk_imlib_create_image_from_drawable"
      end
   
   ext_save(p1, p2, p3: POINTER) is
      external "C"
      alias "gdk_imlib_save_image"
      end
   
   ext_flip_horizontal(p: POINTER) is
      external "C"
      alias "gdk_imlib_flip_image_horizontal"
      end
   
   ext_flip_vertical(p: POINTER) is
      external "C"
      alias "gdk_imlib_flip_image_vertical"
      end
   
   ext_destroy_image(p: POINTER) is
      external "C"
      alias "gdk_imlib_destroy_image"
      end
   
   ext_push_visual(p: POINTER) is
      external "C"
      alias "gtk_widget_push_visual"
      end
   
   ext_push_colormap(p: POINTER) is
      external "C"
      alias "gtk_widget_push_colormap"
      end
   
   ext_get_visual: POINTER is
      external "C"
      alias "gdk_imlib_get_visual"
      end
   
   ext_get_colormap: POINTER is
      external "C"
      alias "gdk_imlib_get_colormap"
      end
   
   ext_init is
      external "C"
      alias "gdk_imlib_init"
      end
   
   ext_load_image(p: POINTER): POINTER is
      external "C"
      alias "gdk_imlib_load_image"
      end
   
   ext_render(p: POINTER; w, h: INTEGER) is
      external "C"
      alias "gdk_imlib_render"
      end
   
   ext_apply(p1, p2: POINTER) is
      external "C"
      alias "gdk_imlib_apply_image"
      end
   
   ext_rgb_width(p: POINTER): INTEGER is
      external "C"
      end
   
   ext_rgb_height(p: POINTER): INTEGER is
      external "C"
      end
   
end


