
indexing
   author: "Miron Brezuleanu"
   purpose: "write the files using the data collected by the wizard"
   
class FILE_GENERATOR
   
inherit
   FILE_TOOLS
   EXG_FILE_TOOLS
   
creation
   make
   
feature
   
   make(pproject_path, pproject_name: STRING;
	puses_timers: BOOLEAN) is
	 -- prepares the object 
      do
	 project_path := pproject_path
	 project_name :=  pproject_name
	 uses_timers := puses_timers
	 exg_path := get_environment_variable("EXG")	 
      ensure
	 exg_path /= Void
      end
   
   run is
	 -- does the actual writing
      do
	 check_path
	 if path_ok then
	    exg_system("install -d " + project_path)
	    write_makefile
	    write_gtk_cecil
	    write_loadpath_se
	    write_class_template
	 end
      end
   
   check_path is
	 -- this will check if the directory name of the project is the
	 -- application name; if not, it will ask a confirmation 
	 -- (this is done to avoid accidentaly writing the files in a 
	 -- wrong location)
      local
	 emb: EXG_MESSAGE_BOX
      do
	 if not project_path.has_suffix(project_name) 
	    and not project_path.has_suffix(project_name+"/") then
	    !!emb.make
	    emb.set_title("Warning")
	    emb.set_message("The directory name is not the %
                            %application name%NAre you sure this is %
                            %what you want?")
	    emb.set_ok_button(False)
	    emb.set_yes_button(True)
	    emb.set_no_button(True)
	    emb.run
	    path_ok := equal(emb.answer, "yes")
	 else
	    path_ok := True
	 end	    
      end
   
   test_overwrite(filename: STRING) is
	 -- test whether the file already exists;
      
	 -- if so, it asks about overwriting
      
	 -- if the answer was "yes" or the file doesn't exist
	 -- attribute `write_ok' is set to `True'
      local
	 emb: EXG_MESSAGE_BOX
      do
	 if is_readable(filename) then 
	    -- maybe there's something better than `is_readable' 
	    -- could be used here
	    !!emb.make
	    emb.set_title("Warning")
	    emb.set_message("File " + filename + " already exists;%N%
						   %overwrite?")
	    emb.set_ok_button(False)
	    emb.set_yes_button(True)
	    emb.set_no_button(True)
	    emb.run
	    write_ok := equal(emb.answer, "yes")
	 else
	    write_ok := True
	 end	 
      end
   
   write_makefile is
	 -- writes the makefile for the new project
      local
	 in_f: STD_FILE_READ
	 out_f: STD_FILE_WRITE
      do
	 test_overwrite(join(<<project_path,"Makefile">>))
	 if write_ok then
	    from
	       !!in_f.connect_to(join(<<exg_path, "/pgen/skel/Makefile">>))
	       !!out_f.connect_to(join(<<project_path, "Makefile">>))
	    until
	       in_f.end_of_input
	    loop
	       in_f.read_line
	       if in_f.last_string.has_prefix("TARGET") then
		  out_f.put_string("TARGET=" + project_name + "%N")
	       elseif in_f.last_string.has_prefix("EIFFEL_GTK") then
		  out_f.put_string("EIFFEL_GTK=$(EXG)/eiffel-gtk%N")
	       else
		  out_f.put_string(in_f.last_string + "%N")
	       end
	    end
	    in_f.disconnect
	    out_f.disconnect
	 end
      end
      
   write_gtk_cecil is
	 -- writes the `gtk_cecil' file for the project
      local
	 in_f: STD_FILE_READ
	 out_f: STD_FILE_WRITE
	 line: STRING
      do
	 test_overwrite(join(<<project_path,"gtk_cecil">>))
	 if write_ok then
	    from
	       !!in_f.connect_to(join(<<exg_path, "pgen/skel/gtk_cecil">>))
	       !!out_f.connect_to(join(<<project_path, "gtk_cecil">>))
	    until
	       in_f.end_of_input
	    loop
	       in_f.read_line
	       if in_f.last_string.has_prefix("--gtk_timer_entry") 
		  and uses_timers then
		  line := clone(in_f.last_string)
		  line.remove_first(2) -- uncomment
		  out_f.put_string(line + "%N")
	       else
		  out_f.put_string(in_f.last_string + "%N")
	       end	    
	    end
	    in_f.disconnect
	    out_f.disconnect
	 end
      end
   
   write_loadpath_se is
	 -- writes the `load_path.se' file for the project
      local
	 src, dest: STRING
      do
	 test_overwrite(join(<<project_path,"load_path.se">>))
	 if write_ok then
	    src := join(<<exg_path, "pgen/skel/loadpath.se">>)
	    dest := join(<<project_path, "loadpath.se">>)
	    exg_system("cp " + src + " " + dest)
	 end
      end
   
   write_class_template is
	 -- writes the minimal Eiffel program
      local
	 in_f: STD_FILE_READ
	 out_f: STD_FILE_WRITE
	 line: STRING
      do
	 test_overwrite(join(<<project_path, project_name+".e">>))
	 if write_ok then
	    from
	       !!in_f.connect_to(join(<<exg_path, "pgen/skel/skel.e">>))
	       !!out_f.connect_to(join(<<project_path, project_name + ".e">>))
	    until
	       in_f.end_of_input
	    loop
	       in_f.read_line
	       if in_f.last_string.has_prefix("class SKEL") then
		  line := clone(project_name)
		  line.to_upper
		  line.prepend("class ")
		  line.append("%N")
		  out_f.put_string(line)
	       else
		  out_f.put_string(in_f.last_string + "%N")
	       end
	    end
	    in_f.disconnect
	    out_f.disconnect
	 end
      end
   
feature -- attributes
   
   exg_path: STRING
   project_path, project_name: STRING
   uses_timers: BOOLEAN   
   write_ok: BOOLEAN   
   path_ok: BOOLEAN
   
end
      
